<?php

namespace Drupal\email_tfa\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a user Email TFA form.
 *
 * @internal
 */
class EmailTfaVerifyForm extends FormBase {

  /**
   * The tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * Constructs a TFA verification form..
   *
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The tempstore factory.
   */
  public function __construct(PrivateTempStoreFactory $temp_store_factory) {
    $this->tempStoreFactory = $temp_store_factory->get('email_tfa');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('user.private_tempstore')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'email_tfa_verify_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['email_tfa_verify'] = [
      '#type' => 'textfield',
      '#title' => $this->t('TFA'),
      '#size' => 60,
      '#required' => TRUE,
    ];
    $form['submit'] = [
      '#type' => 'submit',
      '#name' => 'verify',
      '#value' => $this->t('Verify'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $tfa = $form_state->getValue('email_tfa_verify');
    if ($tfa == $this->tempStoreFactory->get('email_tfa_otp_number') && is_numeric($tfa)) {
      $this->tempStoreFactory->set('email_tfa_user_verify', 1);
      $this->messenger()->addStatus($this->t('Code was successfully authenticated.'));
      // @todo remove Redirect or check where were the user before.
      $url = Url::fromRoute('<front>');
      $form_state->setRedirectUrl($url);
    }
    else {
      $request = \Drupal::request();
      $request->getSession()->clear();
      $url = Url::fromRoute('user.login.http');
      $form_state->setRedirectUrl($url);
      $this->messenger()->addError($this->t('Two-factor Authentication verification failed. Please try again.'));
    }
  }

}
